import dataclasses
from typing import Optional, List
from typing_extensions import Literal

from django_rest_client import (
    APIResponse,
    APIResource,
    RetrievableAPIResourceMixin,
    ListableAPIResourceMixin,
    CreateableAPIResourceMixin,
    PaginationAPIResourceMixin,
)
from django_rest_client.types import Toid, TParams


@dataclasses.dataclass
class CreateAnalysisRequestBody:
    profiles: List[int]
    private: bool = False
    allow_actions: bool = False
    root: bool = False
    os: Optional[Literal["WINDOWS", "LINUX"]] = None
    arguments: Optional[List[str]] = None
    dll_entrypoints: Optional[List[str]] = None


class Analysis(
    APIResource,
    RetrievableAPIResourceMixin,
    ListableAPIResourceMixin,
    CreateableAPIResourceMixin,
    PaginationAPIResourceMixin,
):
    """
    :class:`pydragonfly.Dragonfly.Analysis`
    """

    OBJECT_NAME = "api.analysis"
    EXPANDABLE_FIELDS = {
        "retrieve": ["sample", "reports"],
        "list": [],
    }
    ORDERING_FIELDS = [
        "created_at",
        "sample__filename",
        "weight",
    ]
    CreateAnalysisRequestBody = CreateAnalysisRequestBody

    @classmethod
    def create(
        cls,
        data: CreateAnalysisRequestBody,
        sample_name: str,
        sample_buffer: bytes,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        # first: POST sample uploading it
        resp1 = cls._request(
            "POST",
            url="api/sample",
            files={"sample": (sample_name, sample_buffer)},
        )
        # second: POST analysis using the new sample ID
        # build request body
        req_data = {
            **{k: v for k, v in dataclasses.asdict(data).items() if v is not None},
            "sample_id": resp1.data["id"],
        }
        if resp1.data["malware_type"] == "DLL":
            req_data["dll_entrypoints"] = (
                data.dll_entrypoints
                if data.dll_entrypoints
                else resp1.data["entry_points"]
            )  # dll_entrypoints is required in case of DLL
        resp2 = cls._request(
            "POST",
            url=cls.class_url(),
            json=req_data,
            params=params,
        )
        return resp2

    @classmethod
    def aggregate_evaluations(
        cls,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.class_url() + "/aggregate/evaluations"
        return cls._request("GET", url=url, params=params)

    @classmethod
    def aggregate_status(
        cls,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.class_url() + "/aggregate/status"
        return cls._request("GET", url=url, params=params)

    @classmethod
    def aggregate_malware_families(
        cls,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.class_url() + "/aggregate/malware_families"
        return cls._request("GET", url=url, params=params)

    @classmethod
    def aggregate_malware_type(
        cls,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.class_url() + "/aggregate/malware_type"
        return cls._request("GET", url=url, params=params)

    @classmethod
    def revoke(
        cls,
        object_id: Toid,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.instance_url(object_id) + "/revoke"
        return cls._request("POST", url=url, params=params)



import click
from rich import print as rprint
from pathlib import Path

from pydragonfly import DragonflyException, TParams
from .._utils import (
    ClickContext,
    json_flag_option,
    get_status_text,
    get_success_text,
    add_options,
)
from ._renderables import (
    _display_single_analysis,
    _paginate_table,
    _generate_analysis_table,
)


@click.group("analysis")
def analysis():
    """
    Analysis Management\n
    >>> [API] https://dragonfly.certego.net/api/analysis\n
    >>> [GUI] https://dragonfly.certego.net/history/analysis
    """


@analysis.command("list", help="List all analysis")
@click.option(
    "--status",
    type=click.Choice(
        [
            "QUEUED",
            "PENDING",
            "RUNNING",
            "ANALYZED",
            "FAILED",
            "REVOKED",
        ],
        case_sensitive=False,
    ),
    show_choices=True,
    help="Filter against status",
)
@click.option(
    "--evaluation",
    type=click.Choice(
        [
            "CLEAN",
            "SUSPICIOUS",
            "MALICIOUS",
        ],
        case_sensitive=False,
    ),
    show_choices=True,
    help="Filter against evaluation",
)
@add_options(json_flag_option)
@click.pass_context
def analysis_list(ctx: ClickContext, status: str, evaluation: str, as_json: bool):
    ctx.obj._logger.info("Requesting list of analysis..")
    ctx.obj._logger.info(f"[+] GUI: {ctx.obj._server_url}/history/analysis")
    params = TParams(ordering=["-created_at"])
    if status:
        params["status"] = status.upper()
    if evaluation:
        params["evaluation"] = evaluation.upper()
    try:
        if as_json:
            response = ctx.obj.Analysis.list(params=params)
            rprint(response.data)
        else:
            generator = ctx.obj.Analysis.auto_paging_iter(params=params)
            _paginate_table(generator, _generate_analysis_table)
    except DragonflyException as exc:
        ctx.obj._logger.fatal(str(exc))


@analysis.command("retrieve", help="Retrieve analysis info and result")
@click.argument("analysis_id", type=int)
@add_options(json_flag_option)
@click.pass_context
def analysis_retrieve(ctx: ClickContext, analysis_id: int, as_json: bool):
    ctx.obj._logger.info(f"Requesting analysis [underline blue]#{analysis_id}[/]..")
    try:
        response = ctx.obj.Analysis.retrieve(
            object_id=analysis_id,
            params=TParams(
                expand=["sample"],
                omit=[
                    "sample.sections",
                    "sample.flags",
                    "sample.dlls_imported",
                    "sample.file_version_info",
                ],
            ),
        )
        if as_json:
            rprint(response.data)
        else:
            _display_single_analysis(response.data)
    except DragonflyException as exc:
        ctx.obj._logger.fatal(str(exc))


@analysis.command("create", help="Submit a sample for analysis")
@click.argument("filepath", type=click.Path(exists=True, resolve_path=True))
@click.option(
    "-pl",
    "--profiles-list",
    type=str,
    default="",
    help="Comma separated list of profile indices",
)
@click.option(
    "-os",
    "--operating-system",
    type=click.Choice(["automatic", "WINDOWS", "LINUX"], case_sensitive=False),
    default="automatic",
    show_choices=True,
    show_default=True,
    help="Operating system for profile",
)
@click.option(
    "-al",
    "--arguments-list",
    type=str,
    default="",
    help="Comma separated list of extra command line arguments for the emulator",
)
@click.option(
    "-dl",
    "--dll-entrypoints",
    type=str,
    default="",
    help="""Comma separated list of DLL entrypoints.
    Supports only DLL sample. Default: all entrypoints.
    """,
)
@click.option(
    "-a",
    "--allow-actions",
    is_flag=True,
    show_default=True,
    help="Run actions when a rule matches (coming soon)",
)
@click.option(
    "-r",
    "--root",
    is_flag=True,
    show_default=True,
    help="Emulate with root permissions",
)
@click.option(
    "-p",
    "--private",
    is_flag=True,
    show_default=True,
    help="""
    Mark the analysis as private so it's accessible
    to you and members within your organization only
    """,
)
@click.pass_context
def analysis_create(
    ctx: ClickContext,
    filepath: str,
    profiles_list: str,
    operating_system: str,
    arguments_list: str,
    dll_entrypoints: str,
    allow_actions: bool,
    root: bool,
    private: bool,
):
    fpath = Path(filepath)
    profiles = profiles_list.split(",") if len(profiles_list) else []
    arguments = arguments_list.split(",") if len(arguments_list) else []
    dll_entrypoints = dll_entrypoints.split(",") if len(dll_entrypoints) else []
    try:
        ctx.obj._logger.info(
            f"""
Requesting analysis...
    file: [blue]{fpath.name}[/]
    (   profiles: [i green]{profiles}[/],
        os: [i green]{operating_system}[/],
        private: [i green]{private}[/]
    )
            """
        )
        response = ctx.obj.Analysis.create(
            data=ctx.obj.Analysis.CreateAnalysisRequestBody(
                profiles=profiles,
                os=operating_system
                if operating_system in ["WINDOWS", "LINUX"]
                else None,
                arguments=arguments,
                dll_entrypoints=dll_entrypoints,
                allow_actions=allow_actions,
                root=root,
                private=private,
            ),
            sample_name=fpath.name,
            sample_buffer=fpath.read_bytes(),
        )
        analysis_id = response.data["id"]
        gui_url = response.data["gui_url"]
        status = response.data["status"]
        filename = response.data["sample"]["filename"]
        sha256 = response.data["sample"]["sha256"]
        ctx.obj._logger.info(
            f"""
Success {get_success_text("True", as_text=False)}...
    [+] ID: {analysis_id}
    [+] File/SHA256: [cyan]{filename} ({sha256})[/]
    [+] Status: {get_status_text(status, as_text=False)}
    [+] URL: {gui_url}
            """
        )

    except DragonflyException as exc:
        ctx.obj._logger.fatal(str(exc))



# flake8: noqa
from .action import Action
from .analysis import Analysis
from .invitation import Invitation
from .organization import Organization
from .profile import Profile
from .report import Report
from .rule import Rule
from .sample import Sample
from .session import Session
from .user_access_info import UserAccessInfo
from .user_preferences import UserPreferences



import dataclasses
from typing import Optional, List
from typing_extensions import Literal
from django_rest_client import (
    APIResource,
    APIResponse,
    RetrievableAPIResourceMixin,
    ListableAPIResourceMixin,
    CreateableAPIResourceMixin,
    UpdateableAPIResourceMixin,
    PaginationAPIResourceMixin,
)
from django_rest_client.types import Toid, TParams


@dataclasses.dataclass
class CreateRuleRequestBody:
    rule: str
    weight: int
    modules: dict
    variables: List[str] = dataclasses.field(default_factory=list)
    malware_family: str = ""
    malware_behaviour: Literal[
        "PUA",
        "Riskware",
        "Adware",
        "Greyware",
        "Downloader",
        "Bot",
        "Trojan",
        "Backdoor",
        "Coinminer",
        "Worm",
        "Rootkit",
        "Infostealer",
        "Banker",
        "Ransomware",
        "Spyware",
        "Miner",
        "Unpacker",
    ] = ""
    meta_description: dict = dataclasses.field(default_factory=dict)
    sensitive: bool = False


@dataclasses.dataclass
class UpdateRuleRequestBody:
    enabled: bool


class Rule(
    APIResource,
    RetrievableAPIResourceMixin,
    ListableAPIResourceMixin,
    CreateableAPIResourceMixin,
    UpdateableAPIResourceMixin,
    PaginationAPIResourceMixin,
):
    """
    :class:`pydragonfly.Dragonfly.Rule`
    """

    OBJECT_NAME = "api.rule"
    EXPANDABLE_FIELDS = {
        "retrieve": ["user", "actions", "clause", "permissions"],
        "list": ["user", "permissions"],
    }
    ORDERING_FIELDS = [
        "created_at",
        "rule",
        "weight",
        "malware_family",
        "malware_behaviour",
    ]

    # models
    CreateRuleRequestBody = CreateRuleRequestBody
    UpdateRuleRequestBody = UpdateRuleRequestBody

    @classmethod
    def create(
        cls,
        data: CreateRuleRequestBody,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        post_data = dataclasses.asdict(data)
        return super().create(data=post_data, params=params)

    @classmethod
    def update(
        cls,
        object_id: Toid,
        data: UpdateRuleRequestBody,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        post_data = {
            "enabled": data.enabled,
        }
        return super().update(object_id=object_id, data=post_data, params=params)

    @classmethod
    def aggregate_malware_behaviour(
        cls,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.class_url() + "/aggregate/malware_behaviour"
        return cls._request("GET", url=url, params=params)



from . import APIResourceBaseTestCase, APIResource


class UserAccessInfoResourceTestCase(APIResourceBaseTestCase):
    @property
    def resource(self) -> APIResource:
        return self.df.UserAccessInfo



from . import APIResourceBaseTestCase, APIResource

from tests.mock_utils import (
    generic_200_mock,
    generic_201_mock,
    generic_204_mock,
    if_mock_connections,
    patch,
    MockAPIResponse,
)


class AnalysisResourceTestCase(APIResourceBaseTestCase):
    @property
    def resource(self) -> APIResource:
        return self.df.Analysis

    @if_mock_connections(
        patch(
            "requests.Session.request",
            return_value=MockAPIResponse(
                {"id": 1, "malware_type": "DLL", "entry_points": []}, 201
            ),
        )
    )  # POST /api/sample
    @generic_201_mock  # POST /api/analysis
    def test__create(self, *args, **kwargs):
        response = self.resource.create(
            data=self.resource.CreateAnalysisRequestBody(profiles=[1]),
            sample_name="test.exe",
            sample_buffer=b"",
        )
        self.assertEqual(201, response.code)

    @generic_200_mock
    def test__aggregate_evaluations(self, *args, **kwargs):
        response = self.resource.aggregate_evaluations()
        self.assertEqual(200, response.code)

    @generic_200_mock
    def test__aggregate_status(self, *args, **kwargs):
        response = self.resource.aggregate_status()
        self.assertEqual(200, response.code)

    @generic_200_mock
    def test__aggregate_malware_families(self, *args, **kwargs):
        response = self.resource.aggregate_malware_families()
        self.assertEqual(200, response.code)

    @generic_200_mock
    def test__aggregate_malware_type(self, *args, **kwargs):
        response = self.resource.aggregate_malware_type()
        self.assertEqual(200, response.code)

    @generic_204_mock
    def test__revoke(self, *args, **kwargs):
        response = self.resource.revoke(object_id=self.object_id)
        self.assertEqual(204, response.code)



import click
import click_creds
from rich import print as rprint
from typing import Optional

from ._utils import ClickContext


@click.group("config")
def config():
    """
    Set or view config variables
    """
    pass


@config.command("get")
@click_creds.pass_netrcstore_obj
def config_get(netrc: click_creds.NetrcStore):
    """
    Pretty Print config variables
    """
    rprint(netrc.host_with_mapping)


@config.command("set")
@click.option(
    "-k",
    "--api-key",
    help="Dragonfly API key",
)
@click.option(
    "-c",
    "--certificate",
    type=click.Path(exists=True, resolve_path=True),
    help="Path to SSL client certificate file (.pem)",
)
@click.option(
    "-u",
    "--instance-url",
    default="https://dragonfly.certego.net",
    show_default=True,
    help="Dragonfly's server URL",
)
@click.option(
    "-v",
    "--verify",
    is_flag=True,
    show_default=True,
    help="Boolean determining whether certificate validation is enforced",
)
@click.pass_context
def config_set(
    ctx: ClickContext,
    api_key: Optional[str],
    certificate: Optional[str],
    instance_url: str,
    verify: bool,
):
    """
    Set/Edit config variables
    """
    netrc: click_creds.NetrcStore = click_creds.get_netrc_object_from_ctx(ctx)
    new_host = netrc.host.copy()
    if api_key:
        new_host["password"] = api_key
    if instance_url:
        new_host["account"] = instance_url
    if certificate:
        new_host["login"] = certificate
    if verify is False:
        new_host["login"] = False
    # finally save
    netrc.save(new_host)
    ctx.obj._logger.info("Successfully saved config variables!")



from . import APIResourceBaseTestCase, APIResource

from tests.mock_utils import generic_200_mock, generic_204_mock


class ReportResourceTestCase(APIResourceBaseTestCase):
    @property
    def resource(self) -> APIResource:
        return self.df.Report

    @generic_200_mock
    def test__timeline(self, *args, **kwargs):
        response = self.resource.timeline(object_id=self.object_id)
        self.assertEqual(200, response.code)

    @generic_200_mock
    def test__matched_rules(self, *args, **kwargs):
        response = self.resource.matched_rules(object_id=self.object_id)
        self.assertEqual(200, response.code)

    @generic_204_mock
    def test__revoke(self, *args, **kwargs):
        response = self.resource.revoke(object_id=self.object_id)
        self.assertEqual(204, response.code)



from typing import Optional
import dataclasses
from django_rest_client import (
    APIResource,
    APIResponse,
    CreateableAPIResourceMixin,
    SingletonAPIResourceMixin,
)
from django_rest_client.types import TParams


@dataclasses.dataclass
class CreateOrgRequestBody:
    name: str


@dataclasses.dataclass
class InviteRequestBody:
    username: str


@dataclasses.dataclass
class RemoveMemberRequestBody:
    username: str


class Organization(
    APIResource,
    SingletonAPIResourceMixin,
    CreateableAPIResourceMixin,
):
    """
    :class:`pydragonfly.Dragonfly.Organization`

    Note: ``delete`` and ``leave`` methods are
    intentionally not provided to avoid accidents.
    Please use the GUI for those operations.
    """

    OBJECT_NAME = "api.me.organization"
    EXPANDABLE_FIELDS = {
        "retrieve": ["members", "pending_invitations"],
        "list": ["members", "pending_invitations"],
    }
    ORDERING_FIELDS = []

    # models
    CreateOrgRequestBody = CreateOrgRequestBody
    InviteRequestBody = InviteRequestBody
    RemoveMemberRequestBody = RemoveMemberRequestBody

    @classmethod
    def create(
        cls,
        data: CreateOrgRequestBody,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        post_data = dataclasses.asdict(data)
        return super().create(data=post_data, params=params)

    @classmethod
    def invite(
        cls,
        data: InviteRequestBody,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.class_url() + "/invite"
        return cls._request(
            "POST", url=url, json=dataclasses.asdict(data), params=params
        )

    @classmethod
    def remove_member(
        cls,
        data: RemoveMemberRequestBody,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.class_url() + "/remove_member"
        return cls._request(
            "POST", url=url, json=dataclasses.asdict(data), params=params
        )



import click
from rich import print as rprint

from pydragonfly import DragonflyException
from .._utils import ClickContext


@click.command(help="Get user access info")
@click.pass_context
def access_info(ctx: ClickContext):
    try:
        response = ctx.obj.UserAccessInfo.get()
        rprint(response.data)
    except DragonflyException as exc:
        ctx.obj._logger.fatal(str(exc))



import dataclasses
from typing import List, Optional
from django_rest_client import (
    APIResource,
    APIResponse,
    SingletonAPIResourceMixin,
)
from django_rest_client.mixins import UpdateableAPIResourceMixin
from django_rest_client.types import Toid, TParams


@dataclasses.dataclass
class UpdateUserPreferencesRequestBody:
    apistructure_ignore_list: List[str]


class UserPreferences(
    APIResource,
    UpdateableAPIResourceMixin,
    SingletonAPIResourceMixin,
):
    """
    :class:`pydragonfly.Dragonfly.UserPreferences`
    """

    OBJECT_NAME = "api.me.preferences"
    EXPANDABLE_FIELDS = {
        "retrieve": [],
        "list": [],
    }
    ORDERING_FIELDS = []

    # models
    UpdateUserPreferencesRequestBody = UpdateUserPreferencesRequestBody

    @classmethod
    def update(
        cls,
        object_id: Toid,
        data: UpdateUserPreferencesRequestBody,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        post_data = dataclasses.asdict(data)
        return super().update(object_id=object_id, data=post_data, params=params)



#!/usr/bin/env python3
import click
import click_creds

from pydragonfly import Dragonfly
from .resources import analysis, profile, rule, access_info
from .config import config
from ._utils import (
    get_logger,
    ClickContext,
    get_version_number,
)


@click.group(context_settings=dict(help_option_names=["-h", "--help"]))
@click.option("-d", "--debug", is_flag=True, help="Set log level to DEBUG")
@click.version_option(version=get_version_number())
@click_creds.use_netrcstore(
    name="pydragonfly",
    mapping={"login": "certificate", "password": "api_key", "account": "instance_url"},
)
@click.pass_context
def cli(ctx: ClickContext, debug: bool):
    host = click_creds.get_netrc_object_from_ctx(ctx).host.copy()
    api_key, url = host["password"], host["account"]
    if (not api_key or not url) and ctx.invoked_subcommand != "config":
        click.echo("Hint: Use `config set` to set config variables!")
        exit()
    else:
        logger = get_logger("DEBUG" if debug else "INFO")
        ctx.obj = Dragonfly(api_key=api_key)
        ctx.obj._server_url = url
        ctx.obj._logger = logger


# Compile all groups and commands
for c in [
    config,
    analysis,
    profile,
    rule,
    access_info,
]:
    cli.add_command(c)

# Entrypoint/executor
if __name__ == "__main__":
    cli()



import click
from rich import print as rprint

from pydragonfly import DragonflyException, TParams
from .._utils import (
    ClickContext,
    json_flag_option,
    add_options,
)
from ._renderables import _paginate_table, _generate_profile_table


@click.group("profile")
def profile():
    """
    Profile Management\n
    >>> [API] https://dragonfly.certego.net/api/profile\n
    >>> [GUI] https://dragonfly.certego.net/dashboard/profiles
    """


@profile.command("list", help="List all profiles")
@add_options(json_flag_option)
@click.pass_context
def profile_list(ctx: ClickContext, as_json: bool):
    ctx.obj._logger.info("Requesting list of profiles..")
    ctx.obj._logger.info(f"[+] GUI: {ctx.obj._server_url}/dashboard/profiles")
    params = TParams(ordering=["-created_at"])
    try:
        if as_json:
            response = ctx.obj.Profile.list(params=params)
            rprint(response.data)
        else:
            generator = ctx.obj.Profile.auto_paging_iter(params=params)
            _paginate_table(generator, _generate_profile_table)
    except DragonflyException as exc:
        ctx.obj._logger.fatal(str(exc))


@profile.command("retrieve", help="Retrieve profile object")
@click.argument("profile_id", type=int)
@click.pass_context
def profile_retrieve(ctx: ClickContext, profile_id: int):
    ctx.obj._logger.info(f"Requesting profile [underline blue]#{profile_id}[/]..")
    try:
        response = ctx.obj.Profile.retrieve(
            object_id=profile_id, params=TParams(expand=["user"])
        )
        rprint(response.data)
    except DragonflyException as exc:
        ctx.obj._logger.fatal(str(exc))



from typing import Optional

from django_rest_client import (
    APIResponse,
    APIResource,
    RetrievableAPIResourceMixin,
    ListableAPIResourceMixin,
    PaginationAPIResourceMixin,
)
from django_rest_client.types import Toid, TParams


class Report(
    APIResource,
    RetrievableAPIResourceMixin,
    ListableAPIResourceMixin,
    PaginationAPIResourceMixin,
):
    """
    :class:`pydragonfly.Dragonfly.Report`
    """

    OBJECT_NAME = "api.report"
    EXPANDABLE_FIELDS = {
        "retrieve": ["profile", "analysis", "structs_count"],
        "list": ["profile", "analysis", "structs_count"],
    }
    ORDERING_FIELDS = [
        "time__start_analysis",
        "analysis__sample__filename",
        "weight",
    ]

    @classmethod
    def timeline(
        cls,
        object_id: Toid,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.instance_url(object_id) + "/timeline"
        return cls._request("GET", url=url, params=params)

    @classmethod
    def matched_rules(
        cls,
        object_id: Toid,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.instance_url(object_id) + "/matched-rules"
        return cls._request("GET", url=url, params=params)

    @classmethod
    def revoke(
        cls,
        object_id: Toid,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        url = cls.instance_url(object_id) + "/revoke"
        return cls._request("POST", url=url, params=params)



from . import APIResourceBaseTestCase, APIResource

from tests.mock_utils import generic_200_mock, generic_201_mock


class ProfileResourceTestCase(APIResourceBaseTestCase):
    @property
    def resource(self) -> APIResource:
        return self.df.Profile

    @generic_201_mock
    def test__create(self, *args, **kwargs):
        response = self.resource.create(
            data=self.resource.CreateProfileRequestBody(
                filename="testprofile.ql", emulator="qiling", content=b"test"
            ),
        )
        self.assertEqual(201, response.code)

    @generic_200_mock
    def test__update(self, *args, **kwargs):
        response = self.resource.update(
            object_id=self.object_id,
            data=self.resource.UpdateProfileRequestBody(enabled=False),
        )
        self.assertEqual(200, response.code)



# system imports
from unittest import TestCase
from click.testing import CliRunner

# lib imports
import click_creds
from pydragonfly.cli import cli


class TestConfig(TestCase):
    def setUp(self):
        self.runner: CliRunner = CliRunner()

    def test_config_help(self):
        result = self.runner.invoke(cli, ["config", "--help"])
        self.assertEqual(result.exit_code, 0)
        self.assertIn("Usage", result.output)

    def test_config_get(self):
        result = self.runner.invoke(cli, ["config", "get"])
        self.assertEqual(result.exit_code, 0)

    def test_config_set_and_get(self):
        # step 1
        mapp = {
            "certificate": __file__,
            "api_key": "hax0r",
            "instance_url": "http://localhost",
        }
        result = self.runner.invoke(
            cli,
            [
                "config",
                "set",
                "-c",
                mapp["certificate"],
                "-k",
                mapp["api_key"],
                "-u",
                mapp["instance_url"],
                "-v",
            ],
        )
        self.assertEqual(result.exit_code, 0)
        self.assertIn(
            "Successfully saved config variables!",
            result.output.strip("\n"),
        )
        # step 2
        obj = click_creds.NetrcStore("pydragonfly")
        self.assertDictEqual(mapp, obj.host_with_mapping)



import click
import logging
import json
import datetime as dt
from rich.emoji import Emoji
from rich.text import Text
from rich.syntax import Syntax
from rich.logging import RichHandler

from pydragonfly import Dragonfly

json_flag_option = [
    click.option(
        "-j",
        "--json",
        "as_json",
        is_flag=True,
        help="output as raw JSON",
    ),
]


class ClickContext(click.Context):
    #: ``Dragonfly`` instance
    obj: Dragonfly


def get_status_text(status: str, as_text=True):
    styles = {
        # evalutation
        "clean": ("#73D216", ""),
        "suspicious": ("#CE5C00", ""),
        "malicious": ("#CC0000", ""),
        # statuses
        "queued": ("#CE5C00", str(Emoji("gear"))),
        "pending": ("#CE5C00", str(Emoji("gear"))),
        "running": ("#CE5C00", str(Emoji("gear"))),
        "analyzed": ("#73D216", str(Emoji("heavy_check_mark"))),
        "failed": ("#CC0000", str(Emoji("cross_mark"))),
        "revoked": ("#CC0000", str(Emoji("cross_mark"))),
    }
    color, emoji = styles[status.lower()]
    return (
        Text(status + " " + emoji, style=color)
        if as_text
        else f"[{color}]{status} {emoji}[/]"
    )


def get_success_text(success: str, as_text=True):
    color, emoji = (
        ("#73D216", str(Emoji("heavy_check_mark")))
        if str(success) == "True"
        else ("#CC0000", str(Emoji("cross_mark")))
    )
    return Text(emoji, style=color) if as_text else f"[{color}]{emoji}[/]"


def get_weight_text(weight: int, as_text=True):
    color = "#73D216"
    if weight >= 100:
        color = "#CC0000"
    elif weight >= 50:
        color = "#CE5C00"
    return Text(str(weight), style=color) if as_text else f"[{color}]{weight}[/]"


def get_json_syntax(obj):
    return Syntax(
        json.dumps(obj, indent=2),
        "json",
        theme="ansi_dark",
        word_wrap=True,
        tab_size=2,
    )


def get_datetime_text(str_date: str) -> str:
    return dt.datetime.strptime(str_date, "%Y-%m-%dT%H:%M:%S.%fZ").strftime("%Y-%m-%d")


def get_logger(level: str = "INFO"):
    fmt = "%(message)s"
    logging.basicConfig(
        level=level, format=fmt, datefmt="[%X]", handlers=[RichHandler(markup=True)]
    )
    logger = logging.getLogger("rich")
    return logger


def get_version_number() -> str:
    from ..version import VERSION

    return VERSION


def add_options(options):
    def _add_options(func):
        for option in reversed(options):
            func = option(func)
        return func

    return _add_options



import logging
from django_rest_client import APIClient
from django_rest_client.types import THeaders

from ..version import VERSION
from .resources import (
    Action,
    Analysis,
    Invitation,
    Organization,
    Profile,
    Report,
    Rule,
    Sample,
    Session,
    UserAccessInfo,
    UserPreferences,
)


class Dragonfly(APIClient):
    # overwrite
    _server_url: str = "https://dragonfly.certego.net"

    @property
    def _headers(self) -> THeaders:
        return {
            **super()._headers,
            "User-Agent": f"PyDragonfly/{VERSION}",
        }

    def __init__(self, api_key: str, logger: logging.Logger = None):
        super().__init__(api_key, None, logger)

    # resources
    Action = Action
    Analysis = Analysis
    Invitation = Invitation
    Organization = Organization
    Profile = Profile
    Report = Report
    Rule = Rule
    Sample = Sample
    Session = Session
    UserAccessInfo = UserAccessInfo
    UserPreferences = UserPreferences



import base64
import dataclasses
from typing import Optional
from typing_extensions import Literal
from django_rest_client import (
    APIResource,
    APIResponse,
    RetrievableAPIResourceMixin,
    ListableAPIResourceMixin,
    CreateableAPIResourceMixin,
    UpdateableAPIResourceMixin,
    PaginationAPIResourceMixin,
)
from django_rest_client.types import Toid, TParams


@dataclasses.dataclass
class CreateProfileRequestBody:
    filename: str
    emulator: Literal["qiling", "speakeasy"]
    content: bytes


@dataclasses.dataclass
class UpdateProfileRequestBody:
    enabled: bool


class Profile(
    APIResource,
    RetrievableAPIResourceMixin,
    ListableAPIResourceMixin,
    CreateableAPIResourceMixin,
    UpdateableAPIResourceMixin,
    PaginationAPIResourceMixin,
):
    """
    :class:`pydragonfly.Dragonfly.Profile`
    """

    OBJECT_NAME = "api.profile"
    EXPANDABLE_FIELDS = {
        "retrieve": ["user", "permissions"],
        "list": ["user", "permissions"],
    }
    ORDERING_FIELDS = [
        "id",
        "filename",
        "created_at",
    ]

    # models
    CreateProfileRequestBody = CreateProfileRequestBody
    UpdateProfileRequestBody = UpdateProfileRequestBody

    @classmethod
    def create(
        cls,
        data: CreateProfileRequestBody,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        post_data = {
            "filename": data.filename,
            "emulator": data.emulator,
            "file_buffer_b64": base64.b64encode(data.content),
        }
        return super().create(data=post_data, params=params)

    @classmethod
    def update(
        cls,
        object_id: Toid,
        data: UpdateProfileRequestBody,
        params: Optional[TParams] = None,
    ) -> APIResponse:
        post_data = {
            "enabled": data.enabled,
        }
        return super().update(object_id=object_id, data=post_data, params=params)
